# CS 430 OOP/Testing Lab
# Reference Solution
# Mike Lam
#

require "test/unit"

#############################  BINARY TREE NODE  ##############################

class Node
  attr_accessor :value, :left, :right

  def initialize (value)
    @value = value
    @left = @right = nil
  end

  # binary insertion
  def insert (value)
    @left.insert(value)      if value < @value and not @left.nil?
    @left = Node.new(value)  if value < @value and     @left.nil?
    @right.insert(value)     if value > @value and not @right.nil?
    @right = Node.new(value) if value > @value and     @right.nil?
  end

  # binary search
  def find (value)
    return true if @value == value
    (value < @value) ? (not @left.nil?  and @left.find(value)) :
                       (not @right.nil? and @right.find(value))
  end

  # recursive iteration
  def each (&block)
    @left.each(&block) unless @left.nil?
    block.call(@value)
    @right.each(&block) unless @right.nil?
  end

  # convert to string (infix format)
  def to_s
    ((@left.nil?)  ? "" : @left.to_s) + value.to_s + " " +
    ((@right.nil?) ? "" : @right.to_s)
  end

end


############################  BINARY SEARCH TREE  #############################

class BST
  include Enumerable      # uses each() to provide many other methods
  attr_reader :size

  def initialize
    @root = nil
    @size = 0
  end

  def insert (value)
    if @root.nil?
      @root = Node.new(value)         # first node is a special case
    else
      return if @root.find(value)     # don't add if it's already present
      @root.insert(value)             # recursive insertion
    end
    @size += 1
  end

  def find (value)
    @root.nil? ? false : @root.find(value)  # delegate to root
  end

  def each (&block)
    @root.each(&block) unless @root.nil?    # delegate to root
  end

  def to_s
    @root.nil? ? "" : @root.to_s            # delegate to root
  end

end


################################  UNIT TESTS  #################################

class BST_Test < Test::Unit::TestCase

  def test_node
    n = Node.new("abc")
    assert_equal("abc", n.value)
    assert_equal(nil, n.left)
    assert_equal(nil, n.right)
  end

  def test_BST
    t = BST.new
    assert_equal(0, t.size)
    assert(!t.find("abc"))
    a = []
    t.each { |x| a << x }
    assert_equal([], a)
    assert_equal([], t.to_a)
  end

  def test_BST_single_value
    t = BST.new
    t.insert("abc")
    assert_equal(1, t.size)
    assert(t.find("abc"))
    assert(!t.find("ab"))
    a = []
    t.each { |x| a << x }
    assert_equal(["abc"], a)
    assert_equal(["abc"], t.to_a)
  end

  def test_BST_multiple_values
    t = BST.new
    t.insert("foo")
    t.insert("bar")
    t.insert("bar")
    t.insert("baz")
    t.insert("foobar")
    assert_equal(4, t.size)
    assert(t.find("foo"))
    assert(t.find("bar"))
    assert(t.find("baz"))
    assert(t.find("foobar"))
    assert(!t.find(""))
    assert(!t.find("abc"))
    assert(!t.find("barfoo"))
    assert(!t.find("foofoo"))
    a = []
    t.each { |x| a << x }
    assert_equal(["bar","baz","foo","foobar"], a)
    assert_equal(["bar","baz","foo","foobar"], t.to_a)
    assert_equal(["bar","baz","foo"], t.select{|x|x.size==3})
  end

end

